/*
 *	HID controller 
 *
 *	programmed by piro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include "hidcontroller.h"
#include <signal.h>

#include <qpe/qpeapplication.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/config.h>

#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlcdnumber.h>
#include <qspinbox.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <qslider.h>
#include <qfile.h>
#include <qtextstream.h>

#define MOUSE_L	0x01
#define MOUSE_R 0x02

#define CTRL	0x01
#define SHIFT	0x02
#define ALT	0x04

/* 
 *  Constructs a Example which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
hidcontroller::hidcontroller( QWidget* parent,  const char* name, WFlags fl )
    : hidcontrollerbase( parent, name, fl )
{

  connect(Slider, SIGNAL(valueChanged(int)), this, SLOT(sliderchange(int)));
  connect(loadButton, SIGNAL(clicked()), this, SLOT(moduleLoad()));
  connect(releaseButton, SIGNAL(clicked()), this, SLOT(moduleUnload()));
  connect(lockButton, SIGNAL(clicked()), this, SLOT(keyLock()));
  connect(unlockButton, SIGNAL(clicked()), this, SLOT(keyUnlock()));

  connect(closeButton, SIGNAL(clicked()), this, SLOT(closeClicked()));


  Slider->setValue(75);
  modify = 0;
  mouseButton = 0;

  closeApp = false;

  setCaption( tr( "Hidcontroller" ) );

  
  Config qpeconfig("qpe");
  if (qpeconfig.isValid()){
    qpeconfig.setGroup("Screensaver");
    if (qpeconfig.hasKey("Interval"))
      {
	intervalSuspend = qpeconfig.readNumEntry("Interval", 0);
	intervalLightOff = qpeconfig.readNumEntry("Interval_LightOff", 0);
	intervalDim = qpeconfig.readNumEntry( "Interval_Dim", 0 );
      }
  }

  toggleSuspend(false);

  QFile f("/etc/hotplug/usbd.ftype");
  QString s;
  if ( f.open(IO_ReadOnly) ) {
    QTextStream t( &f );
    s = t.readLine();
    moduleMode->setText(s);
    f.close();
  }

  keyMode->setText("Unlock");
  
  isKHloaded=(fopen("/home/zaurus/Settings/keyhelper_HID.xml", "r") != NULL) ? 1 : 0;
  if (isKHloaded) {
    system("qcop QPE/KeyHelper \"reload(QString)\" keyhelper_HID.xml"); //QcopEnvelope can't use. 
  }

}



/*  
 *  Destroys the object and frees any allocated resources
 */

hidcontroller::~hidcontroller()
{
   // no need to delete child widgets, Qt does it all for us

  toggleSuspend(true);

  releaseKeyboard();
  QPEApplication::ungrabKeyboard();

  if (isKHloaded)
    system("qcop QPE/KeyHelper \"reload()\""); //QcopEnvelope can't use. 

}

void hidcontroller::closeEvent(QCloseEvent *e) {
  if(closeApp)
    e->accept();
  else
    e->ignore();
}

void hidcontroller::closeClicked() {
  if(moduleMode->text() == "HID"){
    return;
  }else {
    closeApp = true;
    close();
  }
}

void hidcontroller::moduleLoad()
{
  char *command = "/home/QtPalmtop/share/hidcontroller/moduleload.sh";

  QFile f("/proc/usbd");
  QString s;
  if ( f.open(IO_ReadOnly) ) {
    QTextStream t( &f );        // use a text stream
    QString s;
    while ( !t.eof() ) {        // until end of file...
      s = t.readLine();       // line of text excluding '\n'
      if (s.find("Cable: Plugged") != -1){
	TextLabel->setText("Cable Plugged!");
	return;
      }
    }
    f.close();
  }

  if(moduleMode->text() == "net"){
    system(command);
    TextLabel->setText("modules loaded");
    moduleMode->setText("HID");
  }else {
    TextLabel->setText("Not net");
  }
}

void hidcontroller::moduleUnload()
{
  char *command = "/home/QtPalmtop/share/hidcontroller/moduleunload.sh";

  QFile f("/proc/usbd");
  QString s;
  if ( f.open(IO_ReadOnly) ) {
    QTextStream t( &f );        // use a text stream
    QString s;
    while ( !t.eof() ) {        // until end of file...
      s = t.readLine();       // line of text excluding '\n'
      if (s.find("Cable: Plugged") != -1){
	TextLabel->setText("Cable Plugged!");
	return;
      }
    }
    f.close();
  }

  if(moduleMode->text() == "HID"){
    system(command);
    TextLabel->setText("modules unloaded");
    moduleMode->setText("net");
  }else {
    TextLabel->setText("Not HID");
  }
}

void hidcontroller::keyUnlock()
{
  keyMode->setText("Unlock");
  releaseKeyboard();
  QPEApplication::ungrabKeyboard();
}

void hidcontroller::keyLock()
{
  QPEApplication::grabKeyboard();
  grabKeyboard();
  keyMode->setText("Lock");
}

void hidcontroller::sliderchange( int val )
{
  QString str;
  str.sprintf("%2.2f",(100 - val)/25.0);
  accel->setText(str);
}

void hidcontroller::mouseMoveEvent( QMouseEvent *e)
{
  QString str;
  int dx,dy;
  int mag;

  if(moduleMode->text() != "HID")
    return;

  if(notvalid)
    return;

  dx = e->globalX() - x;
  dy = e->globalY() - y;

  mag = 100-Slider->value();

  str.sprintf("d %02x %02x, %02x: %d %d slider %d", mouseButton,
	      (char)(dx * mag / 25), (char)(dy * mag / 25),dx*mag,dx*mag/25, mag);


  x = e->globalX();
  y = e->globalY();

  TextLabel->setText(str);

  fp = fopen("/dev/hidout11", "w");
  fprintf(fp, "%03x %03x %03x\n", mouseButton, 
	  (char)(dx * mag / 25), (char)(dy * mag / 25));

  fclose(fp);

}

void hidcontroller::mousePressEvent( QMouseEvent *e)
{
  x = e->globalX();
  y = e->globalY();

  if(x < 100 || x > 550 || y < 20 || y > 250){
    notvalid = true;
  }else{
    notvalid = false;
  }

}

void hidcontroller::QueueKey( QKeyEvent *e, int pressed )
{
  QString str;
  int scancode = e->key();
  unsigned char data;
  unsigned char modifyonce;
  //char senddata[50];

  if(moduleMode->text() != "HID")
    return;

  modifyonce = 0;
  data = 0;
 
  if(pressed){

	/* A - Z */
    if(scancode >= 'A' && scancode <= 'Z'){
      data = scancode - 'A' + 4;

	/* 1 - 9*/
    }else if(scancode >= 0x31 && scancode <= 0x39){
      data = scancode - 0x31 + 0x1e;
    }else {
      switch(scancode) {

	/* 0 */
      case Qt::Key_0:			data = 0x27;				break;

	/* f1 - f10 */
      case 0x200f:			data = 0x3a;				break;
      case 0x2010:			data = 0x3b;				break;
      case 0x2012:			data = 0x3c;				break;
      case 0x2011:			data = 0x3d;				break;
      case 0x201b:			data = 0x3e;				break;
      case 0x200a:			data = 0x3f;				break;
      case 0x200b:			data = 0x40;				break;
      case 0x200c:			data = 0x41;				break;
      case 0x2007:			data = 0x42;				break;
      case 0x2008:			data = 0x43;				break;


	/* fn + */
      case 0x2014:			data = 0x00;				break;  //fn + q
      case Qt::Key_AsciiCircum:		data = 0x23;	modifyonce |= SHIFT;	break;  //fn + w	/* ^ */
      case Qt::Key_Equal:		data = 0x2e;				break;  //fn + e	/* = */
      case Qt::Key_Plus:		data = 0x2e;	modifyonce |= SHIFT;	break;  //fn + r	/* + */
      case Qt::Key_BracketLeft:		data = 0x2f;				break;  //fn + t	/* [ */
      case Qt::Key_BracketRight:	data = 0x30;				break;  //fn + y	/* ] */
      case 0xa2:			data = 0x00;				break;  //fn + u
      case 0xa3:			data = 0x00;				break;	//fn + i
      case 0x2015:			data = 0x00;				break;  //fn + o
      case 0x2016:			data = 0x49;				break;  //fn + p	/* Ins */
      case Qt::Key_CapsLock:		data = 0x39;				break;  //fn + Tab
      case 0x2017:			data = 0x00;				break;	//fn + a
      case 0x2018:			data = 0x35;				break;	//fn + s	/* ` */
      case 0xa5:			data = 0x34;				break;	//fn + d	/* ' */
      case Qt::Key_Backslash:		data = 0x31;				break;  //fn + f,k	/* \ */
      case Qt::Key_Semicolon:		data = 0x33;				break;  //fn + g	/* ; */
      case Qt::Key_Colon:		data = 0x33;	modifyonce |= SHIFT;	break;  //fn + h	/* shift + ; */
      case Qt::Key_Asterisk:		data = 0x25;	modifyonce |= SHIFT;	break;  //fn + j	/* shift + 8 */
      case Qt::Key_Bar:			data = 0x31;	modifyonce |= SHIFT;	break;  //fn + l	/* shift + \ */
      case Qt::Key_Underscore:		data = 0x2d;	modifyonce |= SHIFT;	break;  //fn + b	/* shift + - */
      case 0x2019:			data = 0x00;				break;	//fn + n
      case 0x201a:			data = 0x00;				break;	//fn + m


      case Qt::Key_Control:				modify |= CTRL;		break;
      case 0x5003:   
      case 0x5004:					modify |= ALT;		break;  // kana
      case Qt::Key_Shift:				modify |= SHIFT;	break;

      case 0x5001:
      case 0x5002:			data = 0x35;				break;  // zen		/* ` */		

      case Qt::Key_Minus:		data = 0x2d;				break;
      case Qt::Key_At:			data = 0x1f;	modifyonce |= SHIFT;	break;			/* shift + 2 */

      case Qt::Key_Space:
      case 0x1046:			data = 0x2c;				break;	//fn + space

      case Qt::Key_Comma:		data = 0x38;				break;			/* / */
      case Qt::Key_Less:		data = 0x36;	modifyonce |= SHIFT;	break;			/* shift + , */
      case Qt::Key_Period:		data = 0x37;				break;			/* . */
      case Qt::Key_Greater:		data = 0x36;				break;			/* , */

      case Qt::Key_Left:		data = 0x50;				break;
      case Qt::Key_Up:			data = 0x52;				break;
      case Qt::Key_Right:		data = 0x4f;				break;
      case Qt::Key_Down:		data = 0x51;				break;

      case 0x1010:			data = 0x4a;				break;  //fn + left	/* Home */
      case 0x1011:			data = 0x4d;				break;  //fn + right	/* End */
      case 0x1016:			data = 0x4b;				break;  //fn + up	/* PageUp */
      case 0x1017:			data = 0x4e;				break;  //fn + down	/* PageDown */

      case 0x1007:			data = 0x4c;				break;  //fn + bs	/* del */

      case Qt::Key_Escape:		data = 0x29;				break;
      case Qt::Key_Tab:			data = 0x2b;				break;
      case Qt::Key_Backspace:		data = 0x2a;				break;
      case Qt::Key_Return:		data = 0x28;				break;
      case Qt::Key_Enter:		data = 0x28;				break;


      case 0x103b: // home
	mouseButton |= MOUSE_L;
	fp = fopen("/dev/hidout11", "w");
	fprintf(fp, "%03x 00 00\n", mouseButton);
	fclose(fp);
	break;
      case 0x103a: // menu
	mouseButton |= MOUSE_R;
	fp = fopen("/dev/hidout11", "w");
	fprintf(fp, "%03x 00 00\n", mouseButton);
	fclose(fp);
	break;
      }
    }
  }else {

    switch(scancode){

    case Qt::Key_Shift:		modify &= ~SHIFT;	break;
    case Qt::Key_Control:	modify &= ~CTRL;	break;
    
    case 0x5003:   
    case 0x5004:		modify &= ~ALT;		break;  // kana

    case 0x103b: // home
      mouseButton &= ~MOUSE_L;
      fp = fopen("/dev/hidout11", "w");
      fprintf(fp, "%03x 00 00\n", mouseButton);
      fclose(fp);
      return;
      break;

    case 0x103a: // menu
      mouseButton &= ~MOUSE_R;
      fp = fopen("/dev/hidout11", "w");
      fprintf(fp, "%03x 00 00\n", mouseButton);
      fclose(fp);
      return;
      break;

      
    default: data = 0; break;

    }

  }
  str.sprintf("key %04x  %x :%02x 00 %02x 00 00 00 00 00",scancode, pressed, (modify | modifyonce), data);
  
  
  TextLabel->setText(str);
  
  fp = fopen("/dev/hidout2", "w");
  fprintf(fp, "%03x 00 %03x 00 00 00 00 00\n", (modify | modifyonce), data);
  fclose(fp);
  

  
}


void hidcontroller::toggleSuspend(bool enable)
{
    QCopEnvelope e("QPE/System", "setScreenSaverIntervals(int,int,int)" );
    if (enable)
    {
      e << intervalDim << intervalLightOff << intervalSuspend;
    } 
    else 
      {
	e << 0 << 0 << 0;
      }
}
